import fse from 'fs-extra'
import dircompare from 'dir-compare'
import mkdirp from 'mkdirp'
import moment from 'moment'

import {
	cli,
	log,
	fileAttr,
	FileParser,
	config
} from '../'

export default class FileUtils {

	constructor() {}

	static pathWithRoot(path) {
		if(typeof path === 'undefined' || path === null || path === '') path = ''
		return FileUtils.concatPath(config.root, path.replace(config.root, '')).replace(/\/$/, "")
	}

	static cleanPath(path) {
		if(typeof path !== 'undefined' && path !== null) path = path.replace(/\/$/, "")
		return path
	}

	static concatPath() {
		var path = ''
		Array.prototype.forEach.call([].slice.call(arguments), (argument) => {
			if(path !== '') argument = argument.replace(/^\//, "")
			if(argument !== '') path += FileUtils.cleanPath(argument) + '/'
		})

		path = FileUtils.cleanPath(path)

		return path
	}

	static removeLast(path) {
		return path.substring(0, FileUtils.cleanPath(path).lastIndexOf('/'))
	}

	static replaceExtension(path, ext) {
		return path.substring(0, path.lastIndexOf('.')) + '.' + ext
	}

	static removeExtension(path) {
		return path.substring(0, path.lastIndexOf('.'))
	}

	static filename(path) {
		return FileUtils.cleanPath(path).substring(FileUtils.cleanPath(path).lastIndexOf('/') + 1)
	}

	static isFile(path) {
		try{
			var stat = fse.statSync(path)
      return true
		}catch(e){
			return false
		}
		return false
	}

	static isValidFile(fileName) {
		var dotPosition = fileName.indexOf('.')
		if(dotPosition > 0) return true
		return false
	}

	static cleanTplName(path) {
		var cleanTplName = fileAttr.delete(path)
		cleanTplName = cleanTplName.replace(config.root, '')
		cleanTplName = cleanTplName.split('/')
		cleanTplName.shift()
		return cleanTplName.join('/')
	}

	static cleanFilePath(path) {
		var cleanFilePath = fileAttr.delete(path)
		cleanFilePath = cleanFilePath.replace(config.root, '')
		cleanFilePath = cleanFilePath.split('/')
		cleanFilePath.shift()
		return cleanFilePath.join('/')
	}

  static getFilePath(path) {
  	var res = null
  	if(typeof path !== 'undefined' && path !== null && path !== '') {
  		res = path.replace(config.root)
  		res = FileUtils.concatPath(config.root, config.draft.url, res)
  	}
  	return res
  }

  static getTemplatePath(path) {
  	var res = null
  	if(typeof path !== 'undefined' && path !== null && path !== '') {
  		res = path.replace(config.root)
  		res = FileUtils.concatPath(config.root, config.templates.url, res)
  	}
  	return res
  }

  static deleteOlderRevisionByType(fileName, type) {
  	var folder = fileName.split('/')
  	var file = folder.pop()
  	var extension = file.replace(/.*?\./, '')
  	folder = folder.join('/')
  	var stat = fse.statSync(folder)
  	if(stat){
  		var files = FileParser.getFiles(folder, true, 1, new RegExp("\\." + extension))
  		files.forEach(function (fileItem) {
  			var fname = fileAttr.delete(fileItem.cleanPath)
  			var ftype = fileAttr.get(fileItem.cleanPath).s
  			if(fname === file && ftype === type){
  				var fileDraft = fileItem.path.replace(/-abe-./, '-abe-d')
  				FileParser.removeFile(fileItem.path, FileParser.changePathEnv(fileItem.path, config.data.url).replace(new RegExp("\\." + extension), '.json'))
  				FileParser.removeFile(fileDraft, FileParser.changePathEnv(fileDraft, config.data.url).replace(new RegExp("\\." + extension), '.json'))
  			}
  		})
  	}
  }

  static checkMergedFile(file, merged) {
  	var cleanFilePath = file.cleanFilePath
		var revision = {
  			status: file.status,
  			filePath: file.filePath,
  			date: file.abe_meta.latest.date,
				template: file[config.meta.name].template,
  			cleanFilePath: cleanFilePath
		}
		revision[config.meta.name] = file[config.meta.name]

		if(typeof merged[cleanFilePath] === 'undefined' || merged[cleanFilePath] === null) {
			merged[cleanFilePath] = {}

			merged[cleanFilePath].cleanFilePath = revision.cleanFilePath
			merged[cleanFilePath].date = revision.date
			merged[cleanFilePath].template = revision.template
			merged[cleanFilePath][config.meta.name] = revision[config.meta.name]
			merged[cleanFilePath][file.status] = revision
		}else {
			var oldDate = new Date(merged[cleanFilePath].date)
			var newDate = new Date(revision.date)
			var oldStatus = ''
			if(typeof merged[cleanFilePath][revision.status] !== 'undefined' && merged[cleanFilePath][revision.status] !== null) {
				oldStatus = merged[cleanFilePath][revision.status].status
			}
			var newStatus = revision.status

			// if draft > publish
			if(typeof merged[cleanFilePath][newStatus] === 'undefined' || merged[cleanFilePath][newStatus] === null) {
				merged[cleanFilePath][newStatus] = revision
			}else if(newDate > oldDate && oldStatus === newStatus) {
				merged[cleanFilePath][file.status] = revision
			}
		}
  }
  
  static mergeFiles(files1, files2) {
  	var merged = {}
  	var arMerged = []
		
  	Array.prototype.forEach.call(files1, (file) => {
  		FileUtils.checkMergedFile(file, merged)
  	})

  	Array.prototype.forEach.call(files2, (file) => {
  		FileUtils.checkMergedFile(file, merged)
  	})

  	Array.prototype.forEach.call(Object.keys(merged), (key) => {
  		var merge = merged[key]
  		var publishedDate = (typeof merge.published !== 'undefined' && merge.published !== null) ? new Date(merge.published.date) : null
  		var draftDate = (typeof merge.draft !== 'undefined' && merge.draft !== null) ? new Date(merge.draft.date) : null

  		if(publishedDate !== null && draftDate !== null && publishedDate >= draftDate) {
  			merge.draft = null
  		}
  		var revision = {
  			path: merge.cleanFilePath,
  			template: merge.template,
  			published: merge.published,
  			date: merge.date,
  			draft: merge.draft
  		}
  		arMerged.push(revision)
  	})

  	return arMerged
  }

}
